/*==========================================================================
                      AFUPGC(M,A,B; C,Ab,Bb)

Algebraic number field univariate polynomial greatest common divisor
and cofactors.

Inputs
    M : in Z[x], the minimal polynomial for an algebraic number alpha.
  A,B : in Q(alpha)[x].

Outputs
    C : in Q(alpha)[x], C is monic, C = gcd(A,B).
   Ab : in Q(alpha)[x]. If C not zero then As = A/C else As = 0.
   Bb : in Q(alpha)[x]. If C not zero then Bs = B/C else Bs = 0.
==========================================================================*/
#include "saclib.h"
#include "qepcad.h"

void AFUPGC(M,A,B,C_,Ab_,Bb_)
       Word M,A,B,*C_,*Ab_,*Bb_;
{
       Word C,Ab,Bb,P,A1,A2,R,Cp,d,Q,D,p,qp,Mp,A1p,A2p,Cs,c,Ct,Qt,Dt,t;

Step1: /* A = B = 0. */
       if (A == 0 && B == 0) {
         C = 0;
         Ab = 0;
         Bb = 0;
         goto Return;
       }

Step2: /* A = 0 or B = 0. */
       if (A == 0) {
         C = AFPMON(1,M,B);
         Ab = 0;
         Bb = PINV(0,PLDCF(B),1);
         goto Return;
       }
       if (B == 0) {
         C = AFPMON(1,M,A);
         Bb = 0;
         Ab = PINV(0,PLDCF(A),1);
         goto Return;
       }


       /* General case. */

Step3: /* Initialize. */
       P = PRIME;
       A1 = AFPICR(1,A);
       A2 = AFPICR(1,B);
       IPSRP(2,A1,&R,&A1);
       IPSRP(2,A2,&R,&A2);
       Cs = 0;
       Q = 1;
       D = NIL;
       d = PDEG(B)+1;

Step4: /* Reduce polynomials modulo a prime. */
       if (P == NIL)
	 FAIL("AFUPGCD","Prime list exhausted");
       ADV(P,&p,&P);
       Mp = MPHOM(1,p,M);
       if ( PDEG(Mp) < PDEG(M))  goto Step4;
       A1p = MPHOM(2,p,A1);
       if (PDEG(A1p) < PDEG(A1)) goto Step4;
       A2p = MPHOM(2,p,A2);
       if (PDEG(A2p) < PDEG(A2)) goto Step4;

Step5: /* Compute monic gcd modulo p. */
       Cp = MAFUPGCD(p,Mp,A1p,A2p);
       if (Cp == NIL) goto Step4;

Step6: /* Check degree. */
       c = PDEG(Cp);
       if (c > d)                 /* degree too large, throw prime away */
	 goto Step4;
       if (c == 0) {              /* gcd is 1. */
	 C = PMON(AFFINT(1),0);
	 Ab = A;
	 Bb = B;
	 goto Return;
       }
       if (c < d) {               /* all previous primes were unlucky */
	 Cs = Cp;
	 d = c;
	 Q = p;
	 D = LIST1(p);
	 goto Step4;
       }

Step7: /* Apply Chinese remaindering. */
       if (Q != p) {
	 qp = MDINV(p,MDHOM(p,Q));
	 Qt = IDPR(Q,p);
	 Dt = COMP(p,D);
	 Ct = IPCRA(Q,p,qp,2,Cs,Cp);
       }

Step8: /* Recover rational coefficients if possible. */
       C = AFUPFMRC(Qt,Dt,Ct,Q,D,Cs);
       Q = Qt;
       D = Dt;
       Cs = Ct;
       if (C == NIL) goto Step4;
       
Step9: /* Trial division. */
       AFPQR(1,M,A,C,&Ab,&t);
       if (t == 0) {
	 AFPQR(1,M,B,C,&Bb,&t);
	 if (t != 0) goto Step4;
       } else goto Step4;

Return: /* Prepare for return. */
       *C_ = C;
       *Ab_ = Ab;
       *Bb_ = Bb;
}
