/*===========================================================================
                    SIMPLEQE(A,I,B,J;G,s,t,K,a,b,T1,T2)

Simple extension from a multiple extension, QE version.

Inputs
   A : Z[x].  Minimal polynomial for a real algebraic number alpha.
   I : an acceptable isolating interval for alpha.
   B : Q(alpha)[y].  B(alpha,y) is squarefree defining polynomial for a
       real algebraic number beta.
   J : an acceptable isolating interval for beta.
  
Outputs
   G : Z[x].  Minimal polynomial for a real algebraic number gamma.
              Q(gamma) = Q(alpha,beta).
   s,t : in Z.  gamma = s*alpha + t*beta.
   K : an acceptable isolating interval for gamma.
   a : element of Q(gamma).  a = alpha.
   b : element of Q(gamma).  b = beta.
   T1 : Time for AFUPSFNDB.
   T2 : Time for all else.
===========================================================================*/
#include "saclib.h"
#include "qepcad.h"

void SIMPLEQE(A,I,B,J,G_,s_,t_,K_,a_,b_,T1_,T2_)
       Word A,I,B,J,*G_,*s_,*t_,*K_,*a_,*b_,*T1_,*T2_;
{
       Word G,s,t,K,a,b,T1,T2;
       Word F,F1,Fp,Bt,Is,Js,v,vp;
       Word Gt,g,S,C,Gb,Bp,Bb,Ab,At,x;

Step1: /* Trivial cases. */
       T1 = 0;
       T2 = ACLOCK();                       
       x = AFCR(PMON(RNINT(1),1));
       if (PDEG(A) == 1 && PDEG(B) == 1) {
	  G = PMON(1,1);
	  K = LIST2(0,0); s = 0; t = 0;
	  a = AFFRN(IUPRLP(A));
	  b = AFUPRL(A,B);
	  goto Step7; }
       if (PDEG(A) == 1 && PDEG(B) > 1) {
	  G = AFPICR(1,B);
	  G = PLDCF(PTMV(2,G));
	  IPSRP(1,G,&g,&G);
	  K = J; s = 0; t = 1;
	  a = AFFRN(IUPRLP(A));
	  b = x;
	  goto Step7; }
       if (PDEG(A) > 1 && PDEG(B) == 1) {
	  G = A; K = I; s = 1; t = 0; a = x;
	  b = AFUPRL(A,B);
	  goto Step7; }

Step2: /* Compute squarefree norm. */
       T1 = ACLOCK();
       AFUPSFNDB(A,B,&s,&Bt,&F);
       T1 = ACLOCK() - T1;
       
Step3: /* Find the factor of the norm which has gamma as a root and
	  compute an isolating interval for gamma. */
       Is = I; Js = J; vp = 0;
       do {
          K = RISUM(RIRNP(Is,RNINT(s)),Js);
	  Fp = F; F = NIL;
          while (Fp != NIL) {
	     ADV(Fp,&F1,&Fp);
	     v = IUPVOI(F1,K);
	     if (v > 0) {
		vp = v;
		F = COMP(F1,F); } }
	  if (LENGTH(F) > 1 || vp != 1) {
	     Is = IUPIIR(A,Is);
	     Js = AFUPIIR(A,Is,B,Js); } }
       while (LENGTH(F) > 1 || vp != 1);
       G = FIRST(F);
       
Step4: /* Q(alpha,beta) = Q(alpha). */
       if (PDEG(G) == PDEG(A)) {
          Gt = AFPFIP(1,G);
          AFUPGCDB(A,Gt,Bt,&C,&Gb,&Bb);
          G = A; K = I; s = 1; t = 0; a = x;
          b = PTBCF(1,C);  
          S = AFPROD(A,AFFINT(s),a);
	  b = AFNEG(AFSUM(S,b));
          goto Step7; }
       
Step5: /* gamma = s*alpha + beta. */
       t = 1;
       At = AFPFIP(1,A);
       Bp = AFPICR(1,Bt);
       Bp = PTMV(2,Bp);
       Bp = AFPCR(1,Bp);
       AFUPGCDB(G,At,Bp,&C,&Ab,&Bb);
       a = AFNEG(PTBCF(1,C));
       b = AFDIF(x,AFPROD(G,a,AFFINT(s)));

Step6: /* Make K an acceptable isolating interval. */
       K = IPSIFI(G,K);

Step7: /* Get total time. */
       T2 = ACLOCK() - T2 - T1;

Return: /* Prepare for return. */
       *G_ = G;
       *s_ = s;
       *t_ = t;
       *K_ = K;
       *a_ = a;
       *b_ = b;
       *T1_ = T1;
       *T2_ = T2;
       return;
}
